####################
# Copyright (c) 2010, Perceptive Automation, LLC. All rights reserved.
# http://www.perceptiveautomation.com
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#
# IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
####################

####################
## IMPORTS
import cherrypy
from cherrypy import _cperror

import sqlite3
import datetime
import ConfigParser
import re

from indigopy import indigoconn as ic
from indigopy import indigodb as idb
from indigopy.basereqhandler import BaseRequestHandler, kTrueStr, kFalseStr, kEmptyStr, kTextPageStr, kHtmlPageStr, kXmlPageStr

kSqlConfigFile = "/Library/Application Support/Perceptive Automation/Indigo 4/IndigoSqlClient/IndigoSqlClient.conf"
kPatternFile = "/Library/Application Support/Perceptive Automation/Indigo 4/Preferences/usageReporter-filter_pattern.conf"
kOutputFile = "/var/tmp/usageReporter.txt"

def PluginName():
	return u"Device Usage Report"

def ShowOnControlPageList():
	return True  # if True, then above name/description is shown on the Control Page list indexdef IndigoConnected():
	pass

def PluginDescription():
	return u"Simple list of device usage times in minutes for the selected day."

def IndigoDisconnected():
	pass

class MyRequestHandler(BaseRequestHandler):
	""" Handles HTTP page requests. """
	
	def __init__(self, logFunc, debugLogFunc):
		BaseRequestHandler.__init__(self, logFunc, debugLogFunc)
	
	def index(self):
		thePage = self.getReportForDay()
		return thePage
	index.exposed = True			    # exposed = True must be set for handler func to be called
	
	def getReportForDay(self,inDate=None, dfilter=None, submit=None, filePrint=None):
		# See if we can read the RE from the pattern file. If we can, we will use it and display the filter checkbox on the web page
		try:
			patternFile = open(kPatternFile)
			readPattern = patternFile.readline()
			pattern = re.compile(readPattern)
			foundPattern = 1
		except:
			foundPattern = 0
		if (inDate == None):
			inDate = datetime.date.today() - datetime.timedelta(1)
			dfilter = 1
		totals = dict()
		try:
			config = ConfigParser.ConfigParser()
			config.read(kSqlConfigFile)
			dbType = config.get("client_options", "database_type").lower()
			if dbType == "sqlite":
				dbFileName = config.get("sqlite_client", "database")
				if (len(dbFileName) < 1):
					return "The database file is missing in the SQL logging client so nothing is being logged."
				else:
					dbFileName = "/Library/Application Support/Perceptive Automation/Indigo 4/IndigoSqlClient/" + dbFileName
				deviceChanges = config.get("client_options", "log_device_changes")
				if (deviceChanges == "0"):
					return "Device logging is disabled - you need to configure the SQL logging client to log all device changes."
			elif dbType == "postgres":
				return "This plugin can only use SQLite databases, not PostgreSQL databases."
		except Exception, e:
			return "The plugin can't read the config file for the SQL logging client."
		con = sqlite3.connect(dbFileName, detect_types=sqlite3.PARSE_DECLTYPES|sqlite3.PARSE_COLNAMES)
		con.isolation_level = None
		cur = con.cursor()
		cur.execute(u"select distinct(dev_name) from device_history_basic where date(ts, 'localtime') = date(?) order by dev_name", (unicode(inDate),))
		deviceNameList = cur.fetchall()
		currentDevice = 'starting'
		for row in deviceNameList:
			currentDevice = row[0]
			if dfilter and foundPattern:
					match = pattern.match(currentDevice)
			else:
					match = ""
			if not match:
				cur.execute('select datetime(ts, "localtime") as "[timestamp]", dev_state, dev_value from device_history_basic where dev_name = ? AND date(ts, "localtime") = date(?) order by ts ', (currentDevice, unicode(inDate)))
				deviceActionList = cur.fetchall()
				readPass = 0
				deviceAction = deviceActionList.pop(0)
				deviceActionList.insert(0,deviceAction)
				lastTimestamp = datetime.datetime(deviceAction[0].year,deviceAction[0].month,deviceAction[0].day,0,0,0,0,deviceAction[0].tzinfo)
				lastEvent = 'False'
				totals[currentDevice] = 0
				while len(deviceActionList) > 0:
					readPass += 1 #DEBUGGING
					deviceAction = deviceActionList.pop(0)
					currentTimestamp = deviceAction[0]
					currentEvent = deviceAction[1]
					if lastEvent == 'True' and currentEvent == 'False':
							delta = currentTimestamp - lastTimestamp
							deltaInMinutes = delta.seconds / 60
							totals[currentDevice] += deltaInMinutes
					elif lastEvent == 'False' and currentEvent == 'True':
							lastTimestamp = currentTimestamp
					
					lastEvent = currentEvent
				
				# process any leftovers from the device (I.e. it ended the day True)
				if currentEvent == "True":
					if (datetime.date.today() == datetime.date(deviceAction[0].year,deviceAction[0].month,deviceAction[0].day)):
						endOfDay = datetime.datetime.now()
					else:
						endOfDay = datetime.datetime(deviceAction[0].year,deviceAction[0].month,deviceAction[0].day,23,59,59,0,deviceAction[0].tzinfo)
					delta = endOfDay - lastTimestamp
					deltaInMinutes = delta.seconds / 60
					totals[currentDevice] += deltaInMinutes
		con.close()
		
		totalKeys = totals.keys()
		totalKeys.sort()
		
		if filePrint:
			try:
				outputFile = open(kOutputFile, 'w')
			except IOError:
				# Couldn't open the output file - log an error in the Indigo log
				self._Log("usageReporter: Couldn't open the output file")
			else:
				for currentDevice in totals:
					outputFile.write('\"%s\",%s\n' % (currentDevice, totals[currentDevice]))
				outputFile.close()
		
		# Now, we need to feed the data into the template and render it.
		tmpl = self._GetAndLockPluginTemplate('usageReporter/templates/usageReport.html')
		try:
			tmpl.requestFromPrism = self._IsRequestFromPrism();
			tmpl.totals = totals
			tmpl.totalKeys = totalKeys
			tmpl.inDate = inDate
			tmpl.dfilter = dfilter
			tmpl.dispCheck = foundPattern
			return tmpl.RenderTemplate()
		finally:
			# we ALWAYS release the lock we got on the template or things could get very messy
			tmpl.ReleaseLock();
	
	getReportForDay.exposed = True	  # exposed = True must be set for handler func to be called